#!/usr/bin/env python
# coding: utf-8

import pandas as pd
import numpy as np
import sqlalchemy as sa
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy import select, and_
import config


class FactorDirect(object):
    def __init__(self):
        self.file_name = ['rl_data1.xlsx', 'rl_data2.xlsx']

    def update_direct(self):
        merge_data = []
        for file in self.file_name:
            print('doc/' + file)
            if file == 'rl_data1.xlsx':
                df = pd.read_excel('doc/' + file, skiprows=1)[['因子名称', '因子中文', '类别', '方向']].rename(
                    columns={'因子名称': 'factor_name', '因子中文': 'factor_cn_name', '类别': 'factor_type', '方向': 'direction'})
            else:
                df = pd.read_excel('doc/' + file, skiprows=1)[['因子名', '因子中文名', '因子类别', '方向']].rename(
                    columns={'因子名': 'factor_name', '因子中文名': 'factor_cn_name', '因子类别': 'factor_type', '方向': 'direction'})
            merge_data.append(df)
        merge_data = pd.concat(merge_data)

        DB_URL = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.rl_db_user,
                                                                         config.rl_db_pwd,
                                                                         config.rl_db_host,
                                                                         config.rl_db_port,
                                                                         config.rl_db_database)

        _engine = sa.create_engine(DB_URL)
        _session = sessionmaker(bind=_engine, autocommit=False, autoflush=True)
        _base = automap_base()
        _base.prepare(_engine, reflect=True)

        FactorDetail = _base.classes['factor_detail']
        query = select([FactorDetail]).where(
            and_(
                FactorDetail.flag == 1
            ))
        factor_detail = pd.read_sql(query, _engine)
        factor_detail = factor_detail.drop(['direction'], axis=1)

        loads_data = merge_data[['factor_name', 'direction']].merge(factor_detail, on=['factor_name'],
                                                                    ).drop_duplicates(
            subset=['factor_name'], keep='first').set_index('factor_name')

        for factor_name in loads_data.index.tolist():
            direction = loads_data.loc[factor_name]['direction']
            print(factor_name, direction)
            if np.isnan(direction):
                continue
            session = _session()
            session.execute(
                '''update {0} set flag = 1, direction = {1}, is_verify=1 where factor_name=\'{2}\' and flag=1'''.format(
                    'factor_detail', int(loads_data.loc[factor_name]['direction']), factor_name))
            session.commit()
            session.close()


if __name__ == '__main__':
    factor_direct = FactorDirect()
    factor_direct.update_direct()
